/*
 *	KDE3 Kurses Style (version 0.1)
 *	Copyright (C) 2004, Martin Stubenschrott <stubenschrott@gmx.net>
 *
 *	Based on the great dotNET style written by:
 *		Chris Lee <clee@kde.org>
 *		Carsten Pfeiffer <pfeiffer@kde.org>
 *		Karol Szwed <gallium@kde.org>
 *
 *	This library is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Library General Public
 *	License version 2 as published by the Free Software Foundation.
 *
 *	This library is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	Library General Public License for more details.
 *
 *	You should have received a copy of the GNU Library General Public License
 *	along with this library; see the file COPYING.LIB.  If not, write to
 *	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 */
#include <qstyleplugin.h>
#include <qstylefactory.h>
#include <qpointarray.h>
#include <qpainter.h>
#include <qtabbar.h>
#include <qprogressbar.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qscrollbar.h>
#include <qscrollview.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qdrawutil.h>
#include <qapplication.h>
#include <qvariant.h>
#include <qpixmapcache.h>
#include <qslider.h>
#include <qsettings.h>
#include <kpixmap.h>

#include "kurses.h"
#include "kurses.moc"

/*	This code registers the style with QStyleFactory, which makes it possible
 *	for any code that uses QStyle to find out about it. */
// -- Style Plugin Interface -------------------------
class KursesStylePlugin : public QStylePlugin
{
	public:
		KursesStylePlugin() {}
		~KursesStylePlugin() {}

		QStringList keys() const {
			return QStringList() << "Kurses";
		}

		QStyle* create( const QString& key ) {
			if (key == "kurses")
				return new KursesStyle;
			return 0;
		}
};

Q_EXPORT_PLUGIN( KursesStylePlugin )

// ---------------------------------------------------

KursesStyle::KursesStyle() :
	KStyle( AllowMenuTransparency ),
	kickerMode(false),
	toolbarBlendWidget(0),
	kdeToolbarWidget("kde toolbar widget")
{
	winstyle = 0L;
	winstyle = QStyleFactory::create("Windows");
	
	//	We don't have the Windows style, neither builtin nor as a plugin.
	//	Use any style rather than crashing.
	if (winstyle == 0L)
		winstyle = QStyleFactory::create("B3");

//     if (qApp->inherits("KApplication")) 
//         connect( qApp, SIGNAL( kdisplayPaletteChanged() ), SLOT( paletteChanged() ));

	reverseLayout = QApplication::reverseLayout();
}

KursesStyle::~KursesStyle()
{
}

// stolen completely from dotNET
bool KursesStyle::inheritsKHTML(const QWidget* w) const
{
	if (w && w->parentWidget(true) && w->parentWidget(true)->parentWidget(true) && w->parentWidget(true)->parentWidget(true)->parentWidget(true) &&  w->parentWidget(true)->parentWidget(true)->parentWidget(true)->inherits("KHTMLView"))
		return true;
	else
		return false;
}

void KursesStyle::polish(QWidget* widget)
{
	if (!qstrcmp(qApp->argv()[0], "kicker") || widget->inherits("Kicker"))
		kickerMode = true;

	if (widget->isTopLevel())
		return;

	// for proper toolbar button background
    if ( ::qt_cast<QToolButton*>(widget) || widget->inherits( "QToolBarSeparator" ) ||  widget->inherits( "KToolBarSeparator" ))
    {
        widget->setBackgroundMode(PaletteBackground);
        widget->setBackgroundOrigin(QWidget::WindowOrigin);
    }
	
	// for focus highlight
	if (dynamic_cast<QLineEdit*>(widget) ||
		dynamic_cast<QSpinWidget*>(widget) ||
		dynamic_cast<QScrollView*>(widget))
	{
        widget->installEventFilter(this);
    }

	winstyle->polish(widget);
}

void KursesStyle::unPolish(QWidget* widget)
{
	// for focus highlight
	if (dynamic_cast<QLineEdit*>(widget) ||
		dynamic_cast<QSpinWidget*>(widget) ||
		dynamic_cast<QScrollView*>(widget))
	{
        widget->removeEventFilter(this);
    }

	winstyle->unPolish(widget);
}

/*	This is the code that renders 90+% of the buttons in Kurses.
 *	Changing the code here will most likely affect EVERYTHING.
 *	If you want to modify the style, this is a good place to start.
 *	Also, take a look at the sizeMetric function and play with the
 *	widths that things return for different Frame elements.
 */
void KursesStyle::renderButton(QPainter *p,
                               const QRect &r,
                               const QColorGroup &g,
                               bool sunken) const
{
	if (sunken)
		p->fillRect(r, g.highlight());
	else
		p->fillRect(r, g.button());
}

/*	This is the code that renders the slider handles on scrollbars. It draws
 *	them just like normal buttons, by calling renderButton, but it also adds
 *	very subtle grips to the slider handle.
 */
void KursesStyle::renderSlider(QPainter *p,
                               const QRect &r,
                               const QColorGroup &cg,
							   const bool horiz,
							   const bool down) const
{
	QRect sliderRect = r;
	
	if (horiz)
		sliderRect.addCoords(r.width()/2 - 4, 4, - (r.width()/2 - 4), -4);
	else
		sliderRect.addCoords(4, r.height()/2 - 4, -4, - (r.height()/2 - 4));

	// if the scrollbar is 'in use' draw the whole slider selected
	p->fillRect(r, down ? QBrush(cg.highlight()) : QBrush(cg.midlight()));
	p->fillRect(sliderRect, QBrush(cg.highlight()));
}

/*	This function does the rendering for most of the panels in Kurses.
 *	The QCOORD stuff is used to provide the slightly-rounded corners.
 *	Changing this function will have pretty widespread effects. Also a good
 *	place to start if you're looking to make your own style.
 */
void KursesStyle::renderPanel(QPainter *p,
                              const QRect &r,
                              const QColorGroup &g,
                              bool bright,
                              bool thick,
							  bool enabled,
							  bool /*focused*/) const
{
	if (enabled)
	{
		if (thick) /* draw a border around it */
		{
			p->save();
			p->setPen( g.mid() );
			p->setBrush( bright ? g.midlight() : g.background());
			p->drawRect(r);
			p->restore();
		}
		else
			p->fillRect(r, bright ? g.midlight() : g.background());
	}
	else
			p->fillRect(r, g.background());
	return;
#if 0
	//	Stolen wholesale from dotNET, which stole it from Keramik.
	//	I don't like it, but oh well.
	if (kickerMode)
	{
		if (!thick)
		{
			if (sunken)
			{
				const QCOORD corners[] = { x2, y, x2, y2, x, y2, x, y };
				p->setPen(g.background().dark());
				p->drawConvexPolygon(QPointArray(4, corners));
				p->setPen(g.background().light());
				p->drawPolyline(QPointArray(4, corners), 0, 3);
			}
			else
			{
				const QCOORD corners[] = { x, y2, x, y, x2, y, x2, y2 };
				p->setPen(g.background().dark());
				p->drawPolygon(QPointArray(4, corners));
				p->setPen(g.background().light());
				p->drawPolyline(QPointArray(4, corners), 0, 3);
			}
		}
	}

	p->restore();
#endif
}


void KursesStyle::drawKStylePrimitive(KStylePrimitive kpe,
                                      QPainter *p,
                                      const QWidget* widget,
                                      const QRect &r,
                                      const QColorGroup &cg,
                                      SFlags flags,
                                      const QStyleOption& opt) const
{
	/*int x,y,w,h,x2,y2;
	r.rect (&x,&y,&w,&h);
	x2 = x+w-1; y2 = y+h-1;*/

	switch( kpe )
	{
		//  SLIDER
		//  ------
		case KPE_SliderGroove:
		{
			renderPanel(p, r, cg, true, false, flags & Style_Enabled);
			break;
		}
		case KPE_SliderHandle:
		{
			p->fillRect(r, (flags & Style_Enabled) ? cg.highlight() : cg.mid().light(105));
			break;
		}

		default:
			KStyle::drawKStylePrimitive(kpe, p, widget, r, cg, flags, opt);
	}
}


// This function draws primitive elements as well as their masks.
void KursesStyle::drawPrimitive(PrimitiveElement pe,
                                QPainter *p,
                                const QRect &r,
                                const QColorGroup &cg,
                                SFlags flags,
                                const QStyleOption &opt ) const
{
	bool down   = flags & Style_Down;
	bool on     = flags & Style_On;
	bool horiz  = flags & Style_Horizontal;
	bool enabled  = flags & Style_Enabled;
	bool focused  = flags & Style_HasFocus;

	int x, x2, y, y2, w, h;
	r.rect(&x, &y, &w, &h);
	r.coords(&x, &y, &x2, &y2);

	switch(pe)
	{
		case PE_HeaderSection:
		{
			p->fillRect(r, QBrush(cg.midlight()));
			break;
	   	}

		//	BUTTONS
		//	-------
		case PE_ButtonTool:
		{
			p->fillRect(r, (down | on ) ? cg.highlight() : cg.background());
			break;
		}
		case PE_ButtonBevel: 
		case PE_ButtonDropDown:
		case PE_ButtonCommand:
		{
			if (down)
			{
				renderButton(p, r, cg, true);
				p->setPen( cg.highlightedText() );
			}
			else if (!enabled)
				p->fillRect(r, cg.background());
			else
				renderButton(p, r, cg, false);

			break;
		}
		case PE_ButtonDefault:
		{
			p->setPen(cg.highlight());
			p->drawRect(r);
			break;
		}

		//	SCROLLBAR
		//	---------
		case PE_ScrollBarSlider:
		{
			renderSlider(p, r, cg, horiz, down);
			break;
		}
		case PE_ScrollBarAddPage:
		case PE_ScrollBarSubPage:
		{
			if (on || down)
				p->fillRect(r, QBrush(cg.mid().dark()));
			else
				p->fillRect(r, QBrush(cg.midlight()));
			break;
		}
		case PE_ScrollBarAddLine:
		case PE_ScrollBarSubLine:
		{
			if (on || down)
				p->fillRect(r, QBrush(cg.highlight()));
			else
				p->fillRect(r, QBrush(cg.midlight()));
			
			p->setPen(down ? cg.highlightedText() : cg.foreground());
			if (pe == PE_ScrollBarAddLine)
				drawPrimitive((horiz ? PE_ArrowRight : PE_ArrowDown), p, r, cg, flags);
			else
				drawPrimitive((horiz ? PE_ArrowLeft : PE_ArrowUp), p, r, cg, flags);

			break;
		}

		//	CHECKBOXES
		//	----------
		case PE_Indicator:
		{
			renderPanel(p, r, cg, true, false/*inheritsKHTML(opt.widget())*/, flags & Style_Enabled);
			break;
		}
		case PE_IndicatorMask:
		{
			p->fillRect (r, color1);
			break;
		}

		//	RADIOBUTTONS
		//	------------
		case PE_ExclusiveIndicator:
		{
			p->setBrush(flags & Style_Enabled ? cg.base() : cg.mid());
			if (false/*inheritsKHTML(opt.widget())*/)
				p->setPen(cg.mid());
			else
				p->setPen( NoPen );
			p->drawEllipse(r);
			break;
		}
		case PE_ExclusiveIndicatorMask:
		{
			p->fillRect(r, color0);
			p->setPen(color1);
			p->setBrush(color1);
			p->drawEllipse(r);
			break;
		}

	//	GENERAL PANELS
	//	--------------

		// default winstyle group boxes
		case PE_PanelGroupBox:
		case PE_GroupBoxFrame:
			p->save();
			p->setBrush (NoBrush);
			p->setPen (cg.mid().light(105));
			p->drawRect(r);
			p->restore();
			break;

		case PE_WindowFrame:
		case PE_Panel:
		{
			renderPanel(p, r, cg, true, false, enabled, focused);
			break;
		}

		case PE_PanelLineEdit:
		{
			renderPanel(p, r, cg, true, false/*inheritsKHTML(opt.widget())*/, enabled, focused);
			break;
		}

		case PE_StatusBarSection:
		{
			// non FLAT style:
			// renderPanel(p, r, cg, true, false, false, false);
			break;
		}

		case PE_TabBarBase:	// Still not sure what this one does
		case PE_PanelTabWidget:
		{
			break;
		}

		case PE_PanelPopup: // XXX: what is this doing?
		{
			p->setPen(cg.midlight().light(105));
			p->drawLine(r.topLeft(), r.topRight());
			p->drawLine(r.topLeft(), r.bottomLeft());
			p->setPen(cg.dark());
			p->drawLine(r.topRight(), r.bottomRight());
			p->drawLine(r.bottomLeft(), r.bottomRight());
			break;
		}

		//	MENU / TOOLBAR PANEL
		//	--------------------
		case PE_PanelMenuBar: 			// Menu
		case PE_PanelDockWindow:		// Toolbar
		{
			p->fillRect(r, cg.background());
			break;
		}

		//	TOOLBAR/DOCK WINDOW/SPLITTER HANDLE
		//	-----------------------------------
		case PE_Splitter: 
		/* fall through */
		case PE_DockWindowHandle:
		{
			p->fillRect(r, cg.background());
			break;
		}

		/* also used for the gripper of scroll bars */
		case PE_DockWindowResizeHandle:
		{
			p->fillRect(r, cg.button());
#if 0

			int j = r.center().x();
			int k = r.center().y();
			if (horiz)
			{
				p->setPen(cg.dark());
				p->drawLine(j - 2, k - 6, j + 3, k - 6);
				p->drawLine(j - 2, k    , j + 3, k	);
				p->drawLine(j - 2, k + 6, j + 3, k + 6);

				p->setPen(cg.mid().dark(110));
				p->drawLine(j - 1, k - 3, j + 2, k - 3);
				p->drawLine(j - 1, k + 3, j + 2, k + 3);
			}
			else
			{
				p->setPen(cg.dark());
				p->drawLine(j - 6, k - 2, j - 6, k + 3);
				p->drawLine(j    , k - 2, j    , k + 3);
				p->drawLine(j + 6, k - 2, j + 6, k + 3);

				p->setPen(cg.mid().dark(110));
				p->drawLine(j - 3, k - 1, j - 3, k + 2);
				p->drawLine(j + 3, k - 1, j + 3, k + 2);
			}
#endif
			break;
		}


		//	TOOLBAR SEPARATOR
		//	-----------------
		case PE_DockWindowSeparator:
		{
#if 0
			QPen dotPen = cg.dark();
			p->setPen( dotPen );

			int j = r.center().x();
			int k = r.center().y();
			if (horiz)
				for (int i=y+1; i <= y+h-1; i+=3)
					p->drawPoint(j, i);
			else
				for (int i=x+1; i <= x+w-1; i+=3)
					p->drawPoint(i, k);
#endif
			break;
		}

		case PE_CheckMark:
		{
			QPen pen = p->pen();
			if (flags & Style_Enabled)
				pen.setColor( cg.highlight() );
			else
				pen.setColor( cg.mid() );

			if (flags & Style_Down) // if mouse button is down, must be before Style_On
				p->fillRect(r, flags & Style_Down ? cg.highlight() : pen.color());
			else if (flags & Style_On)
				p->fillRect(x+3,y+3,w-6,h-6, flags & Style_Down ? cg.highlight() : pen.color());
			else if (flags & Style_Off) { } // empty
			else	//tristate
				p->fillRect (x+3,y+3,w-6,h-6, cg.highlight().light(105));

			break;
		}

		case PE_SpinWidgetUp:
		case PE_SpinWidgetDown:
		case PE_HeaderArrow:
		case PE_ArrowUp:
		case PE_ArrowDown:
		case PE_ArrowLeft:
		case PE_ArrowRight:
		{
			QPointArray a;

			switch (pe)
			{
				case PE_SpinWidgetUp:
				case PE_ArrowUp:
				{
					a.setPoints(7, u_arrow);
					break;
				}
				case PE_SpinWidgetDown:
				case PE_ArrowDown:
				{
					a.setPoints(7, d_arrow);
					break;
				}
				case PE_ArrowLeft:
				{
					a.setPoints(7, l_arrow);
					break;
				}
				case PE_ArrowRight:
				{
					a.setPoints(7, r_arrow);
					break;
				}
				default:
				{
					if (flags & Style_Up) 
						a.setPoints(7, u_arrow);
					else 
						a.setPoints(7, d_arrow);
				}
			}

			p->save();

			if (flags & Style_Down)
				p->translate(pixelMetric(PM_ButtonShiftHorizontal), pixelMetric(PM_ButtonShiftVertical));

			a.translate((r.x() + r.width()/2), (r.y() + r.height()/2));
			if (p->pen() == QPen::NoPen) 
			{
				if (flags & Style_Enabled)
					p->setPen(cg.buttonText());
				else
					p->setPen(cg.mid());
			}

			p->drawLineSegments(a, 0, 3);
			p->drawPoint(a[6]);

			p->restore();
			break;
		}

		default:
			return winstyle->drawPrimitive(pe, p, r, cg, flags, opt);
	}
}


void KursesStyle::drawControl(ControlElement element,
                              QPainter *p,
                              const QWidget *widget,
                              const QRect &r,
                              const QColorGroup &cg,
                              SFlags flags,
                              const QStyleOption& opt) const
{
	switch (element)
	{
		//	CHECKBOXES
		//	----------
		case CE_CheckBox:
		{
			drawPrimitive(PE_Indicator, p, r, cg, flags);
			drawPrimitive(PE_CheckMark, p, r, cg, flags);
			break;
		}
		case CE_CheckBoxLabel:
		case CE_RadioButtonLabel:
		{
			const QButton *button = static_cast<const QButton *>(widget);
			drawItem( p, r, AlignLeft|AlignVCenter|ShowPrefix, button->colorGroup(),
					button->isEnabled(), button->pixmap(), button->text());
			break;
		}

		//	PROGRESSBAR
		//	-----------
		case CE_ProgressBarGroove:
		{
			//XXX:renderPanel(p, r, cg, true, false, flags & Style_Enabled);
			p->fillRect(r, cg.midlight());
			break;
		}

		case CE_ProgressBarContents:
		{
			const QProgressBar *pb = (const QProgressBar *)widget;

			if (pb->totalSteps())
			{
				int x, y, w, h;
				double percent = (double)pb->progress() / (double)pb->totalSteps();

				r.rect(&x, &y, &w, &h);

				/* draw the background of the progress bar */
				//XXX:renderPanel(p, r, cg, true, false, flags & Style_Enabled);
				p->fillRect(r, cg.midlight());

				if (reverseLayout)
				{
					x += w - ((int)(w * percent));
					w = ((int)(w * percent));
				}
				else
					w = (int)(w * percent);

				// draw the actual progress
				p->fillRect(x, y, w, h, cg.highlight());
			}
			break;
		}

		//	RADIOBUTTONS
		//	------------
		case CE_RadioButton:
		{
			drawPrimitive(PE_ExclusiveIndicator, p, r, cg, flags);

			if (flags & Style_On || flags & Style_Down)
			{
				QCOORD center[] = { 4, 5, 4, 7, 5, 8, 7, 8, 8, 7, 8, 5, 7, 4, 5, 4 };
				QPointArray c(8, center);
				p->setPen(cg.highlight());
				p->setBrush(cg.highlight());
				if (flags & Style_Down)
					p->drawEllipse(r);
				else
					p->drawPolygon(c);
			}
			break;
		}

		//	TABS
		//	----
		case CE_TabBarTab:
		{
			bool selected = flags & Style_Selected;
			p->fillRect(r, selected ? cg.highlight() : cg.mid().light(105));
			break;
		}

		case CE_TabBarLabel:
		{
			const int text_flags = AlignVCenter | AlignHCenter | ShowPrefix | DontClip | SingleLine;

			p->save();
			if (flags & Style_Selected) /* XXX: seems not to work correctly in konqueror */
				p->setPen(flags & Style_Enabled ? cg.highlightedText() : cg.mid().dark());
			else
				p->setPen(flags & Style_Enabled ? cg.foreground() : cg.mid().dark());
			p->drawText(r, text_flags, opt.tab()->text());

			p->restore();
			break;
		}

		case CE_PushButton:
		{
			QPushButton *button = (QPushButton *)widget;
			QRect br = r;
			bool btnDefault = button->isDefault();

			// offset all buttons by 1 pixel on each side, so we can draw the default
			// button indicator for default buttons
			static int di = pixelMetric(PM_ButtonDefaultIndicator);
			br.addCoords(di, di, -di, -di);

			if (btnDefault)
				drawPrimitive(PE_ButtonDefault, p, r, cg, flags);

			drawPrimitive(PE_ButtonBevel, p, br, cg, flags);
			break;
		}


        case CE_PushButtonLabel:
        {
            int x, y, w, h;
            r.rect( &x, &y, &w, &h );

            const QPushButton* button = static_cast<const QPushButton *>( widget );
            bool active = button->isOn() || button->isDown();
            bool cornArrow = false;

            // Shift button contents if pushed.
            if ( active )
            {
                x += pixelMetric(PM_ButtonShiftHorizontal, widget);
                y += pixelMetric(PM_ButtonShiftVertical, widget);
                flags |= Style_Sunken;
            }

            // Does the button have a popup menu?
            if ( button->isMenuButton() )
            {
                int dx = pixelMetric( PM_MenuButtonIndicator, widget );
                if ( button->iconSet() && !button->iconSet()->isNull()  &&
                    (dx + button->iconSet()->pixmap (QIconSet::Small, QIconSet::Normal, QIconSet::Off ).width()) >= w )
                {
                    cornArrow = true; //To little room. Draw the arrow in the corner, don't adjust the widget
                }
                else
                {
                    drawPrimitive( PE_ArrowDown, p, visualRect( QRect(x + w - dx - 8, y + 2, dx, h - 4), r ),
                                cg, flags, opt );
                    w -= dx;
                }
            }

            // Draw the icon if there is one
            if ( button->iconSet() && !button->iconSet()->isNull() )
            {
                QIconSet::Mode  mode  = QIconSet::Disabled;
                QIconSet::State state = QIconSet::Off;

                if (button->isEnabled())
                    mode = button->hasFocus() ? QIconSet::Active : QIconSet::Normal;
                if (button->isToggleButton() && button->isOn())
                    state = QIconSet::On;

                QPixmap pixmap = button->iconSet()->pixmap( QIconSet::Small, mode, state );

                if (button->text().isEmpty() && !button->pixmap())
                    p->drawPixmap( x + w/2 - pixmap.width()/2, y + h / 2 - pixmap.height() / 2, pixmap );
                else
                    p->drawPixmap( x + 4, y + h / 2 - pixmap.height() / 2, pixmap );

                if (cornArrow) //Draw over the icon
                    drawPrimitive( PE_ArrowDown, p, visualRect( QRect(x + w - 6, x + h - 6, 7, 7), r ),
                                cg, flags, opt );


                int  pw = pixmap.width();
                x += pw + 4;
                w -= pw + 4;
            }

			drawItem( p, QRect(x, y, w, h), AlignCenter|ShowPrefix, button->colorGroup(),
						button->isEnabled(), button->pixmap(), button->text());


#if 0
            if ( flags & Style_HasFocus )
                drawPrimitive( PE_FocusRect, p,
                                visualRect( subRect( SR_PushButtonFocusRect, widget ), widget ),
                                cg, flags );
#endif
            break;
        }

		//	MENUBAR ITEM (sunken panel on mouse over)
		//	-----------------------------------------
		case CE_MenuBarItem:
		{
			QMenuItem *mi = opt.menuItem();
			bool active  = flags & Style_Active;
			bool focused = flags & Style_HasFocus;
			const int text_flags = AlignVCenter | AlignHCenter | ShowPrefix | DontClip | SingleLine;

			if (active)
				p->fillRect(r, cg.highlight());
			else
				p->fillRect(r, cg.background());

			p->setPen((active && focused ? cg.highlightedText() : cg.buttonText()));
			p->drawText(r, text_flags, mi->text());
			break;
		}


		//	POPUPMENU ITEM (highlighted on mouseover)
		//	------------------------------------------
		case CE_PopupMenuItem:
		{
			QMenuItem *mi = opt.menuItem();
			if (!mi)
				return;

			const QPopupMenu *pum = (const QPopupMenu *) widget;
			static const int itemFrame = 2;
			static const int itemHMargin = 3;
			static const int itemVMargin = 3;
			static const int rightBorder = 20;
			const int tab = opt.tabWidth();

			int checkcol = QMAX(opt.maxIconWidth(), 20);
			int x, x2, y, y2, w, h;
			r.rect(&x, &y, &w, &h);
			r.coords(&x, &y, &x2, &y2);

			bool active = flags & Style_Active;
			bool disabled = !mi->isEnabled();
			bool checkable = pum->isCheckable();
			bool etchtext = styleHint (SH_EtchDisabledText);

			QColor textcolor; // color of the menu item's text

			int xpos = x;
			int xm = itemFrame + checkcol + itemHMargin;

			if (pum->erasePixmap() && !pum->erasePixmap()->isNull())
				p->drawPixmap( x, y, *pum->erasePixmap(), x, y, w, h );
			else
			{
				QPalette pal = pum->palette();

				int dim = pixelMetric(PM_MenuButtonIndicator) + 4;
				if (!reverseLayout)
					p->fillRect( x, y, x + w - dim, h, cg.midlight());
				else 
					p->fillRect( x+dim, y, w - dim, h, cg.midlight());
			}

			/* don't draw any seperators */
			if (mi->isSeparator())
				return;

			if (active && !disabled)
			{
				p->setBrush(cg.highlight());
				p->fillRect(x, y, w, h, cg.highlight());
				p->setPen(NoPen);
				
				const QCOORD corners[] = { x+1, y2-1, x+1, y+1, x2-1, y+1, x2-1, y2-1 };
				p->drawConvexPolygon(QPointArray(4, corners));
				p->setPen(cg.highlightedText());
			}

			if (mi->iconSet())
			{
				p->save();

				QIconSet::Mode mode = disabled ? QIconSet::Disabled : QIconSet::Normal;
				QPixmap pixmap = mi->iconSet()->pixmap(QIconSet::Small, mode);
				int pixw = pixmap.width();
				int pixh = pixmap.height();

				QRect cr(xpos, y, opt.maxIconWidth(), h);
				QRect pmr(0, 0, pixw, pixh);
				pmr.moveCenter(cr.center());

				if (reverseLayout)
					pmr = visualRect( pmr, r );

				p->setPen(cg.highlightedText());
				p->drawPixmap(pmr.topLeft(), pixmap);

				p->restore();
			}

			p->setPen(active ? cg.highlightedText() : cg.buttonText());

			xpos += xm;

			if (mi->custom())
			{
				int m = itemVMargin;
				p->setPen(cg.foreground());
				mi->custom()->paint(p, cg, active, !disabled, x+xm, y+m, w-xm-tab+1, h-2*m);
				return;
			}
			else
			{
				QString s = mi->text();
				if(!s.isNull())
				{
					int t = s.find('\t');
					int m = itemVMargin;

					int text_flags = AlignVCenter | ShowPrefix | DontClip | SingleLine;

					// set our text color
					if (active && !disabled)
						textcolor = cg.highlightedText();
					else if (disabled)
						textcolor = cg.mid().dark(110);
					else
						textcolor = cg.buttonText();

					// we have a shortcut on the right side of the menu
					if (t >= 0)
					{
						int xp;
						xp = x + w - tab - rightBorder - itemHMargin - itemFrame + 1;

						QRect rr = QRect(xp, y+m, tab, h-(2*m));
						if (reverseLayout)
							rr = visualRect(rr, r);

						//draw etched text if needed
						if (etchtext && disabled)
						{
							QRect etchedRect = QRect(rr);
							etchedRect.addCoords(1,1,1,1);
							p->setPen(cg.light());
							p->drawText(etchedRect, text_flags, s.mid(t+1));
						}

						p->setPen(textcolor);
						p->drawText(rr, text_flags, s.mid(t+1));
						s = s.left(t);
					}

					QRect rr = QRect(xpos, y+m, w-xm-tab+1, h-(2*m));
					if (reverseLayout) {
						rr = visualRect(rr, r);
						text_flags |= AlignRight;
					}

					//draw etched text if needed
					if (etchtext && disabled)
					{
						QRect etchedRect = QRect(rr);
						etchedRect.addCoords(1,1,1,1);
						p->setPen(cg.light());
						p->drawText(etchedRect, text_flags, s);
					}

					p->setPen(textcolor);
					p->drawText(rr, text_flags, s);
				}
				else if (mi->pixmap())
				{
					QPixmap *pixmap = mi->pixmap();
					if (pixmap->depth() == 1) 
						p->setBackgroundMode(OpaqueMode);
					p->drawPixmap(xpos, y, *pixmap);
					if (pixmap->depth() == 1)
						p->setBackgroundMode(TransparentMode);
				}
			}

			// a submenu
			if (mi->popup())
			{
				int dim = pixelMetric(PM_MenuButtonIndicator);

				xpos = x+w - dim - 2;

				if (active && !disabled)
					p->setPen(cg.highlightedText());
				else
					p->setPen(cg.text());


				QRect rr = QRect(xpos, y + h/2 - dim/2, dim, dim);
				if (reverseLayout)
					rr = visualRect( rr, r );
				drawPrimitive((reverseLayout ? PE_ArrowLeft : PE_ArrowRight), p, rr, cg, Style_Enabled);
			}

			if (checkable)
			{
				if (mi->isChecked())
				{
					int xp = x;

					SFlags cflags = Style_Default;

					cflags |= Style_On;
					if (!disabled)
						cflags |= Style_Enabled;

					QRect rr = QRect( xp + 3, y + 3, 16, h - 5 );
					if (reverseLayout)
						rr = visualRect( rr, r );

					drawPrimitive(PE_CheckMark, p, rr, cg, cflags);
				}
			}

			break;
		}

		// Menu and dockwindow/toolbar empty space
		//
		case CE_DockWindowEmptyArea:
		case CE_MenuBarEmptyArea:
			p->fillRect(r, cg.background());
			break;

		default:
			winstyle->drawControl(element, p, widget, r, cg, flags, opt);
	}
}

/* XXX: can remove this override once qstyle does not paint etched text */
void KursesStyle::drawItem( QPainter *p, const QRect &r,
               int flags, const QColorGroup &g, bool enabled,
               const QPixmap *pixmap, const QString& text, int len,
               const QColor* penColor ) const
{
	int x = r.x();
    int y = r.y();
    int w = r.width();
    int h = r.height();

    p->setPen( penColor ? *penColor : enabled ? g.foreground() : g.mid() );
    if ( pixmap )
	{
    QPixmap  pm( *pixmap );
    bool clip = (flags & Qt::DontClip) == 0;
    if ( clip ) {
        if ( pm.width() < w && pm.height() < h ) {
        clip = FALSE;
        } else {
        p->save();
        QRegion cr = QRect(x, y, w, h);
        if (p->hasClipping())
            cr &= p->clipRegion(QPainter::CoordPainter);
        p->setClipRegion(cr);
        }    
    }    
    if ( (flags & Qt::AlignVCenter) == Qt::AlignVCenter )
        y += h/2 - pm.height()/2;
    else if ( (flags & Qt::AlignBottom) == Qt::AlignBottom)
        y += h - pm.height();
    if ( (flags & Qt::AlignRight) == Qt::AlignRight )
        x += w - pm.width();
    else if ( (flags & Qt::AlignHCenter) == Qt::AlignHCenter )
        x += w/2 - pm.width()/2;
    else if ( ((flags & Qt::AlignLeft) != Qt::AlignLeft) && QApplication::reverseLayout() ) // AlignAuto && rightTo>
        x += w - pm.width();

    if ( !enabled ) {
        if ( pm.mask() ) {          // pixmap with a mask
        if ( !pm.selfMask() ) {     // mask is not pixmap itself
            QPixmap pmm( *pm.mask() );
            pmm.setMask( *((QBitmap *)&pmm) );
            pm = pmm; 
        }
        } else if ( pm.depth() == 1 ) { // monochrome pixmap, no mask
        pm.setMask( *((QBitmap *)&pm) );
#ifndef QT_NO_IMAGE_HEURISTIC_MASK
        } else {                // color pixmap, no mask
        QString k;
        k.sprintf( "$qt-drawitem-%x", pm.serialNumber() );
        QPixmap *mask = QPixmapCache::find(k);
        bool del=FALSE;
        if ( !mask ) {
            mask = new QPixmap( pm.createHeuristicMask() );
            mask->setMask( *((QBitmap*)mask) );
            del = !QPixmapCache::insert( k, mask );
        }
        pm = *mask;
        if (del) delete mask;
#endif
        }
    }    
    p->drawPixmap( x, y, pm );
    if ( clip )
        p->restore();
    }    

    p->drawText( x, y, w, h, flags, text, len );
}

 
 
void KursesStyle::drawComplexControl(ComplexControl control,
                                     QPainter *p,
                                     const QWidget *widget,
                                     const QRect &r,
                                     const QColorGroup &cg,
                                     SFlags flags,
                                     SCFlags controls,
                                     SCFlags active,
                                     const QStyleOption& opt) const
{
	bool enabled = flags & Style_Enabled;
	bool focused = flags & Style_HasFocus;

	switch(control)
	{
		//	LISTVIEW
		//	--------
		case CC_ListView:
		{
			if ( controls & SC_ListView )
				QCommonStyle::drawComplexControl( control, p, widget, r, cg, flags, controls, active, opt );

			if ( controls & ( SC_ListViewBranch | SC_ListViewExpand ) )
			{
				if (opt.isDefault())
					break;

				QListViewItem	*item = opt.listViewItem(),
								*child = item->firstChild();

				int y = r.y();
				int c;
				if ( active == SC_All && controls == SC_ListViewExpand ) 
					c = 2;
				else
				{
					int linebot = 0;

					// skip the stuff above the exposed rectangle
					while ( child && y + child->height() <= 0 )
					{
						y += child->totalHeight();
						child = child->nextSibling();
					}

					int bx = r.width() / 2;

					// paint stuff in the magical area
					QListView* v = item->listView();
					while ( child && y < r.height() )
					{
						int lh;
						if ( !item->multiLinesEnabled() )
							lh = child ? child->height() : 0;
						else
							lh = p->fontMetrics().height() + 2 * v->itemMargin();
						lh = QMAX( lh, QApplication::globalStrut().height() );
						if ( lh % 2 > 0 )
							lh++;
						linebot = y + lh/2;

						if ( (child->isExpandable() || child->childCount()) && (child->height() > 0) )
						{
							p->setPen(cg.dark());
							QPointArray a;
							if ( !child->isOpen() ) // closed
								a.setPoints(7, r_arrow);
							else //or open
								a.setPoints(7, d_arrow);

							a.translate(bx, linebot);
							p->drawLineSegments(a, 0, 3);
							p->drawPoint(a[6]);
						}
						y += child->totalHeight();
						child = child->nextSibling();
					}
				}
				p->setPen(cg.text());
			}
			break;
		}

	
		//	COMBOBOX
		//	--------
		case CC_ComboBox:
		{
			int x, y, w, h;
			const QComboBox *cb = (const QComboBox *)widget;
			r.rect(&x, &y, &w, &h);

			if (active & Style_Sunken)
				flags |= Style_Sunken;

			static const unsigned char downarrow_bits[] =
			{
				0x7f, 0xbe, 0x9c, 0x08, 0x00, 0x00, 0x00, 0x28,
				0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0xb8
			};

			static const unsigned int handle_width = 16;
			static const unsigned int handle_offset = handle_width + 1;

			QBitmap downArrow = QBitmap(7, 4, downarrow_bits, true);
			downArrow.setMask(downArrow);

			int fw = pixelMetric(PM_DefaultFrameWidth, widget);
			if (flags & Style_Enabled)
			{
				if (fw >= 1) /* in khtml mode only */
					renderPanel(p, r, cg, true, true, true);

				cb->editable() ? p->fillRect(x + fw, y + fw, w - 2*fw, h - 2*fw, cg.base()) :
					p->fillRect(x + fw, y + fw, w - 2*fw, h - 2*fw, cg.midlight());
			}
			else
				p->fillRect(x + fw, y + fw, w - 2*fw, h - 2*fw, cg.background());

			// draw the arrow depending whether the combo box is opened or not
			bool opened = cb->listBox() ? cb->listBox()->isVisible() : false;

			QRect cr(w - handle_offset + 1, (opened ? y + 1 : y), handle_width, h - 2);
			QRect pmr(0, 0, 7, 4);
			pmr.moveCenter(cr.center());
			if (reverseLayout) {
				pmr = visualRect( pmr, r );
			}

			drawPrimitive((opened ? PE_ArrowDown : PE_ArrowLeft), p, pmr, cg, Style_Enabled);

			// draw focus rectangle
			if (focused)
			{
				QRect focusRect = visualRect( subRect( SR_ComboBoxFocusRect, widget ), widget );
				drawPrimitive( PE_FocusRect, p, focusRect, cg, flags);
			}

			break;
		}
		
		//  SLIDER
		//  ------
		case CC_Slider:
		{
			QRect groove = visualRect (querySubControlMetrics(CC_Slider, widget, SC_SliderGroove, opt), widget);
			QRect handle = visualRect (querySubControlMetrics(CC_Slider, widget, SC_SliderHandle, opt), widget);

			// draw the 'base' (=groove) of the slider
			if ((controls & SC_SliderGroove) && groove.isValid())
			{
				drawKStylePrimitive ( KPE_SliderGroove, p, widget, groove, cg, flags );

				// draw focus rectangle
				if (flags & Style_HasFocus)
				{
					QRect focusRect = visualRect( subRect( SR_SliderFocusRect, widget ), widget );
					drawPrimitive( PE_FocusRect, p, focusRect, cg, flags);
				}
			}

			// and the handle
			if ((controls & SC_SliderHandle) && handle.isValid())
                 drawKStylePrimitive ( KPE_SliderHandle, p, widget, handle, cg, flags );

			// NOTE: we don't draw any tickmarks
			break;
		}

		//	TOOLBUTTON
		//	----------
		case CC_ToolButton:
		{
			const QToolButton *tb = (const QToolButton *) widget;
#if 1
			QRect button, menuarea;
			button   = querySubControlMetrics(control, widget, SC_ToolButton, opt);
			menuarea = querySubControlMetrics(control, widget, SC_ToolButtonMenu, opt);

			SFlags bflags = flags,
			       mflags = flags;

			if (active & SC_ToolButton)
				bflags |= Style_Down;

			if (active & SC_ToolButtonMenu)
				mflags |= Style_Down;

			if (controls & SC_ToolButton)
			{
				//	If we're pressed, on, or raised...
				if (bflags & (Style_Down | Style_On | Style_Raised))
				{
					drawPrimitive(PE_ButtonTool, p, button, cg, bflags, opt);
				}
				else if (tb->parentWidget() && tb->parentWidget()->backgroundPixmap() &&
				           !tb->parentWidget()->backgroundPixmap()->isNull())
				{
					QPixmap pixmap = *(tb->parentWidget()->backgroundPixmap());
					p->drawTiledPixmap( r, pixmap, tb->pos() );
				}
			}

			// Draw a toolbutton menu indicator if required
			if (controls & SC_ToolButtonMenu)
			{
				if (mflags & (Style_Down | Style_On | Style_Raised)) 
					drawPrimitive(PE_ButtonDropDown, p, menuarea, cg, mflags, opt);

				drawPrimitive(PE_ArrowDown, p, menuarea, cg, mflags, opt);
			}

			if (tb->hasFocus() && !tb->focusProxy())
			{
				QRect fr = tb->rect();
				fr.addCoords(2, 2, -2, -2);
				drawPrimitive(PE_FocusRect, p, fr, cg);
			}
#endif
			break;
		}

		//	SPINWIDGETS
		//	-----------
		case CC_SpinWidget:
		{
			const QSpinWidget *sw = (const QSpinWidget *) widget;
			SFlags flags;
			PrimitiveElement pe;

			QRect swf = querySubControlMetrics(control, sw, SC_SpinWidgetFrame);
			QRect swbf = querySubControlMetrics(control, sw, SC_SpinWidgetButtonField);

			if (controls & SC_SpinWidgetFrame) // Don't draw the WindowsStyle thick frame.
				renderPanel(p, swf, cg, true, true, enabled, focused);

			if (controls & SC_SpinWidgetButtonField) 
			{
				p->save();

				p->setPen (NoPen);
				p->setBrush((sw->isEnabled()) ? cg.base() : cg.background());
				p->drawRect(swbf);

				p->restore();
			}

			if (controls & SC_SpinWidgetUp)
			{
				flags = Style_Default | Style_Enabled;
				if (active == SC_SpinWidgetUp)
				{
					flags |= Style_On;
					flags |= Style_Sunken;
				}
				else
					flags |= Style_Raised;

				if (sw->buttonSymbols() == QSpinWidget::PlusMinus)
					pe = PE_SpinWidgetPlus;
				else
					pe = PE_SpinWidgetUp;

				QRect re = sw->upRect();
				QColorGroup ucg = sw->isUpEnabled() ? cg : sw->palette().disabled();
				QColor col;
				if (!(sw->isEnabled()))
					col = ucg.background();
				else if (flags & Style_Raised)
					col = ucg.base();
				else
					col = ucg.highlight();
					
				p->fillRect(re.x(), re.y(), re.width(), re.height(), col);
				re.setX(re.x() + 1);
				re.setY(re.y() + 1);
				p->setPen(flags & Style_Raised ? ucg.foreground() : ucg.highlightedText());
				drawPrimitive(pe, p, re, ucg, flags);
			}

			if (controls & SC_SpinWidgetDown)
			{
				flags = Style_Default | Style_Enabled;
				if (active == SC_SpinWidgetDown)
				{
					flags |= Style_On;
					flags |= Style_Sunken;
				}
				else
					flags |= Style_Raised;

				if (sw->buttonSymbols() == QSpinWidget::PlusMinus)
					pe = PE_SpinWidgetMinus;
				else
					pe = PE_SpinWidgetDown;

				QRect re = sw->downRect();
				QColorGroup dcg = sw->isDownEnabled() ? cg : sw->palette().disabled();
				QColor col;
				if (!(sw->isEnabled()))
					col = dcg.background();
				else if (flags & Style_Raised)
					col = dcg.base();
				else
					col = dcg.highlight();
					
				p->fillRect(re.x(), re.y(), re.width(), re.height(), col);
				re.setX(re.x() + 1);
				re.setY(re.y() + 1);
				p->setPen(flags & Style_Raised ? dcg.foreground() : dcg.highlightedText());
				drawPrimitive(pe, p, re, dcg, flags);
			}

			break;
		}

		default:
			KStyle::drawComplexControl(control, p, widget, r, cg, flags, controls, active, opt);
			break;
	}
}


QRect KursesStyle::subRect(SubRect r, const QWidget *widget) const
{
	switch (r)
	{
		case SR_ComboBoxFocusRect:
		{
			QRect focusRect = querySubControlMetrics( CC_ComboBox, widget, SC_ComboBoxEditField );
			//focusRect.addCoords(1,1,-1,-1);
			return focusRect;
		}
		case SR_SliderFocusRect:
		{
			QRect focusRect = widget->rect();
			focusRect.addCoords(1,1,-1,-1);
			return focusRect;
		}

		case SR_ProgressBarGroove:
		case SR_ProgressBarLabel:
		{
			return winstyle->subRect(r, widget);
		}
		case SR_ProgressBarContents:
			return widget->rect(); // no border around progress

		default: {
			return KStyle::subRect(r, widget);
		}
	}
}

QRect KursesStyle::querySubControlMetrics(ComplexControl control,
                                          const QWidget *widget,
                                          SubControl subcontrol,
                                          const QStyleOption &opt) const
{
	if (!widget)
		return QRect();

	QRect r(widget->rect());
	switch (control)
	{
		case CC_ComboBox:
		{
			switch (subcontrol)
			{
				case SC_ComboBoxEditField: 
					return QRect(r.x() + 2, r.y() + 2, r.width() - 19, r.height() - 4);

				case SC_ComboBoxListBoxPopup: 
				{
					QRect list = opt.rect();
					list.addCoords(1,-1,-1,-1);
					return list;
				}

				default: 
					return KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
			}
			break;
		}

		case CC_SpinWidget:
		{
			int fw = pixelMetric(PM_SpinBoxFrameWidth, widget);
			QSize bs;
			bs.setHeight(QMAX(8, widget->height()/2));
			bs.setWidth(12);

			int y = fw;
			int x, lx, rx;
			x = widget->width() - y - bs.width();
			lx = fw;
			rx = x - fw;

			switch (subcontrol)
			{
				case SC_SpinWidgetUp: 
					return QRect(x, y+1, bs.width(), bs.height());

				case SC_SpinWidgetDown:
					return QRect(x, y + bs.height()-1, bs.width(), bs.height()-1);

				case SC_SpinWidgetFrame:
					return QRect(0, 0, widget->width(), widget->height());

				case SC_SpinWidgetEditField: 
					return QRect(lx, fw, widget->width() - (bs.width() + fw), widget->height() - 2 * fw);

				case SC_SpinWidgetButtonField:
					return QRect(x, y, bs.width(), widget->height() - 2 * fw);
			}
			break;
		}

		/*case CC_ToolButton:
		{
			switch (subcontrol)
			{
				case SC_ToolButton:
					return QRect(0, 0, widget->width(), widget->height());
			}
			break;
		}*/

		case CC_Slider:
		{
			switch (subcontrol)
			{
				case SC_SliderHandle:
					QRect handle = KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
					//handle.addCoords(1,1,-1,-1);
					return handle;
			}
		}
		
		default:
		{
		}
	}
	return KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
}

int KursesStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
{
	switch(m)
	{
		//	TABS
		//	----
		case PM_TabBarTabVSpace:
		{
			const QTabBar * tb = (const QTabBar *) widget;
			if (tb->shape() == QTabBar::RoundedAbove ||
			    tb->shape() == QTabBar::RoundedBelow)
				return 4;
			else
				return 0;
		}
		case PM_TabBarBaseOverlap:
			return -1;
		case PM_TabBarTabOverlap:
			return 0;

		//	SPLITTERS
		//	---------
		case PM_SplitterWidth:
		case PM_DockWindowHandleExtent:
			return 2;

		//	PROGRESSBARS
		//	------------
		case PM_ProgressBarChunkWidth:
			return 9;

		//	SLIDER
		//	------
		case PM_SliderLength:
			return 16;

		//	MENU INDICATOR
		//	--------------
		case PM_MenuButtonIndicator:
			return 8;

		//	HEADER INDICATOR
		//	----------------
		case PM_HeaderMarkSize:
		case PM_HeaderGripMargin:
			return 5;

		//	CHECKBOXES / RADIO BUTTONS
		//	--------------------------
		case PM_ExclusiveIndicatorWidth:    // Radiobutton size
		case PM_ExclusiveIndicatorHeight:   // 13x13
		case PM_IndicatorWidth:             // Checkbox size
		case PM_IndicatorHeight:            // 13x13
			return 13;

		//	FRAMES
		//	------
		case PM_SpinBoxFrameWidth:
		case PM_DockWindowFrameWidth:
		case PM_MenuBarFrameWidth:
			return 0;

		case PM_DefaultFrameWidth:
		{
			// on HTML pages, a border around text fields really helps identifying them
			/* XXX:if (widget && inheritsKHTML(widget))
				return 1;
			else*/
				return 0;
		}

		case PM_ButtonDefaultIndicator: 
			return 0;
		case PM_MenuBarItemSpacing: // pixels between File, Edit, ...
		case PM_ButtonShiftVertical:
		case PM_ButtonShiftHorizontal:
			return 0;

		// after this amount, jump back to the original scroll bar position
		case PM_MaximumDragDistance:
			return 200;

		default:
			return KStyle::pixelMetric(m, widget);
	}
}


QSize KursesStyle::sizeFromContents(ContentsType t,
                                    const QWidget *widget,
                                    const QSize &s,
                                    const QStyleOption &opt) const
{
	switch (t)
	{
		case CT_PopupMenuItem:
		{
			if (!widget || opt.isDefault())
				break;

			const QPopupMenu *popup = (const QPopupMenu *)widget;
			QMenuItem *mi = opt.menuItem();
			int maxpmw = opt.maxIconWidth();
			int w = s.width(), h = s.height();
			bool checkable = popup->isCheckable();

			if (mi->custom())
			{
				w = mi->custom()->sizeHint().width();
				h = mi->custom()->sizeHint().height();
			}
			else if (mi->widget())
			{
				// don't change the size in this case.
			}
			else if (mi->isSeparator())
			{
				w = 12;
				h = 12;
			}
			else
			{
				if (mi->pixmap()) 
					h = QMAX(h, mi->pixmap()->height() + 2);

				h = QMAX(h, widget->fontMetrics().height() + 4);

				if (mi->iconSet()) 
					h = QMAX(h, mi->iconSet()->pixmap(QIconSet::Small, QIconSet::Normal).height() + 2); //8
			}

			if (!mi->text().isNull())
			{
				if ((mi->text().find('\t') >= 0) || mi->popup()) 
					w += 20;
			}

			if (maxpmw) 
				w += maxpmw;

			if (checkable && maxpmw < 20) 
				w += 20 - maxpmw;

			if (checkable || maxpmw > 0) 
				w += 2;

			w += 20;

			return QSize(w, h);
		}

		case CT_PushButton:
	    {
			const QPushButton* btn = static_cast<const QPushButton*>(widget);

			int w = s.width() + 2 * pixelMetric(PM_ButtonMargin, widget);
			int h = s.height() + 2 * pixelMetric(PM_ButtonMargin, widget);
			if ( btn->isDefault() || btn->autoDefault() )
			{
				w += 2* PM_ButtonDefaultIndicator;
				h += 2* PM_ButtonDefaultIndicator;
			}
			if ( btn->text().isEmpty() && s.width() < 32 )
				return QSize(w, h);

			return QSize( w+5, h );
		}

		case CT_ComboBox:
		{
			int arrow = 21;
			int fw = pixelMetric(PM_DefaultFrameWidth, widget);
			return QSize( s.width() + arrow, QMAX(s.height() + 2*fw, 16));
		}

		default:
			return KStyle::sizeFromContents (t, widget, s, opt);
	}

	return KStyle::sizeFromContents (t, widget, s, opt);
}

int KursesStyle::styleHint(StyleHint stylehint,
						   const QWidget *widget,
						   const QStyleOption &option,
						   QStyleHintReturn* returnData ) const
{
	int ret;

	switch (stylehint)
	{
		case SH_Slider_SnapToValue:
		case SH_PrintDialog_RightAlignButtons:
		case SH_ItemView_ChangeHighlightOnFocus:
		case SH_FontDialog_SelectAssociatedText:
		case SH_PopupMenu_AllowActiveAndDisabled:
		case SH_MenuBar_MouseTracking:
		case SH_PopupMenu_MouseTracking:
		case SH_ComboBox_ListMouseTracking:
		case SH_ScrollBar_MiddleClickAbsolutePosition:
		case SH_UnderlineAccelerator:
		case SH_MenuBar_AltKeyNavigation:
			ret = 1;
			break;

		case SH_EtchDisabledText:
		case SH_MainWindow_SpaceBelowMenuBar:
		case SH_PopupMenu_SubMenuPopupDelay:
		case SH_ToolButton_Uses3D:
			ret = 0;
			break;

		case SH_ScrollBar_BackgroundMode:
			ret = NoBackground;
			break;

		case SH_TabBar_Alignment:
			return AlignHCenter;

		default:
			ret = QCommonStyle::styleHint(stylehint, widget, option, returnData);
			break;
	}

	return ret;
}

bool KursesStyle::eventFilter(QObject *object, QEvent *event)
{
	if (KStyle::eventFilter( object, event))
		return true;

	if ( !object->isWidgetType() )
		return false;

	// focus highlight
    if ( ::qt_cast<QLineEdit*>(object) ||
		 ::qt_cast<QScrollView*>(object))
	{
        QWidget* widget = static_cast<QWidget*>(object);

        if ((event->type() == QEvent::FocusIn) || (event->type() == QEvent::FocusOut))
        {
#if 0
            widget->repaint(false);
#endif
        }
        return false;
    }

	return false;
}

// vim: noet ts=4 sw=4
