/*
 *
 *	Copyright 2004, Martin Stubenschrott <stubenschrott@gmx.net>
 *	Originally copied from the dotNET style, modified to fit mine.
 *
 *	This library is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Library General Public
 *	License version 2 as published by the Free Software Foundation.
 *
 *	This library is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	Library General Public License for more details.
 *
 *	You should have received a copy of the GNU Library General Public License
 *	along with this library; see the file COPYING.LIB.  If not, write to
 *	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 */

#ifndef __KURSES_H
#define __KURSES_H

#include <kstyle.h>
#include <qbitmap.h>

#define u_arrow -4,1, 2,1, -3,0, 1,0, -2,-1, 0,-1, -1,-2
#define d_arrow -4,-2, 2,-2, -3,-1, 1,-1, -2,0, 0,0, -1,1
#define l_arrow 0,-3, 0,3,-1,-2,-1,2,-2,-1,-2,1,-3,0
#define r_arrow -2,-3,-2,3,-1,-2, -1,2,0,-1,0,1,1,0

#define QCOORDARRLEN(x) sizeof(x)/(sizeof(QCOORD)*2)

class KursesStyle : public KStyle
{
	Q_OBJECT

public:
	KursesStyle();
	virtual ~KursesStyle();

	bool inheritsKHTML( const QWidget* widget ) const;
	void polish( QWidget* widget );
	void unPolish( QWidget* widget );

	void drawKStylePrimitive( KStylePrimitive kpe,
		QPainter* p,
		const QWidget* widget,
		const QRect &r,
		const QColorGroup &cg,
		SFlags flags = Style_Default,
		const QStyleOption& = QStyleOption::Default ) const;

	void drawPrimitive( PrimitiveElement pe,
		QPainter *p,
		const QRect &r,
		const QColorGroup &cg,
		SFlags flags = Style_Default,
		const QStyleOption &opt = QStyleOption::Default ) const;

	void drawControl( ControlElement element,
		QPainter *p,
		const QWidget *widget,
		const QRect &r,
		const QColorGroup &cg,
		SFlags flags = Style_Default,
		const QStyleOption& = QStyleOption::Default ) const;

	void drawItem( QPainter *p, const QRect &r,
               int flags, const QColorGroup &g, bool enabled,
               const QPixmap *pixmap, const QString& text, int len = -1,
               const QColor* penColor = 0) const;

	void drawComplexControl( ComplexControl control,
		QPainter *p,
		const QWidget *widget,
		const QRect &r,
		const QColorGroup &cg,
		SFlags flags = Style_Default,
		SCFlags controls = SC_All,
		SCFlags active = SC_None,
		const QStyleOption& = QStyleOption::Default ) const;

	int pixelMetric( PixelMetric m,
		const QWidget *widget = 0 ) const;

	QRect subRect( SubRect r,
		const QWidget *widget ) const;

	QRect querySubControlMetrics( ComplexControl control,
		const QWidget *widget,
		SubControl subcontrol,
		const QStyleOption &opt = QStyleOption::Default ) const;

	QSize sizeFromContents(QStyle::ContentsType t,
	                       const QWidget *w,
	                       const QSize &s,
	                       const QStyleOption &o) const;

	
//                                    const QWidget *widget = NULL,
//                                    const QStyleOption &option = QStyleOption::Default,
//                                    QStyleHintReturn* returnData = NULL ) const;

protected:
	void renderButton(QPainter *p,
	                  const QRect &r,
	                  const QColorGroup &g,
	                  bool sunken = false) const;

	void renderPanel(QPainter *p,
	                 const QRect &r,
	                 const QColorGroup &g,
	                 bool bright = true, /* draw it with a white background */
	                 bool thick = true,
					 bool enabled = true,
					 bool focused = false) const;

	void renderSlider(QPainter *p,
	                  const QRect &r,
	                  const QColorGroup &g,
					  const bool horiz = false,
					  const bool down = false) const;

	void renderStripedBackground(QPainter *p,
	                  const QRect &r,
	                  const QColorGroup &cg) const;
	
	/*void renderToolbarWidgetBackground(QPainter* p, const QWidget* widget) const;
	void renderToolbarEntryBackground(	QPainter* p,
										const QToolBar* parent,
										QRect r,
										const QColorGroup& cg,
										bool horiz) const;
*/
    bool eventFilter(QObject *, QEvent *);
    int styleHint( StyleHint stylehint,
					   const QWidget *widget = NULL,
					   const QStyleOption &option = QStyleOption::Default,
					   QStyleHintReturn* returnData = NULL) const;
	
//     void updatePalette( QToolBar * );
//     void updatePalette( QMenuBar * );

protected slots:
//     void slotDestroyed();
//     void paletteChanged();

private:
	// Disable copy constructor and = operator
	KursesStyle( const KursesStyle & );
	KursesStyle& operator=( const KursesStyle & );
	QStyle *winstyle;

	bool reverseLayout, kickerMode;
	mutable const QWidget *toolbarBlendWidget;
	const char* kdeToolbarWidget;

//     QValueList<QWidget*> m_widgets;
};

#endif

// vim: set noet ts=4 sw=4:
